/**
 * Application
 */
import React, { Fragment, useRef, useEffect, useState } from "react";
import { BryntumDemoHeader, BryntumThemeCombo, BryntumGantt } from "@bryntum/gantt-react";
import { Toast } from "@bryntum/gantt";
import { ganttConfig } from "./AppConfig";
import "./App.scss";
import { getParamUrl } from "./util";
import DemoButton from './components/DemoButton';

const App = () => {
    const ganttComponent = useRef();
    let gantt = null;
    let syncIds = [];

    // edit button click handler
    const handleEditClick = ({ record, grid : gantt }) => {
        // check if the record is available in selectedRecords
        const isSelectedItem = gantt.selectedRecords.find(task => task.id
=== record.id);
        // if selectedRecords doesn"t contain record
        if (!isSelectedItem) {
            // add the record to selectedRecords
            gantt.selectedRecords = [ ...gantt.selectedRecords, record ];
        }
        gantt.editTask(record);
    };

    //Events
    useEffect(() => {
        let loaded = false;
        gantt = ganttComponent.current.instance;

        //Validate add more fields, change color before send to server
        gantt.project.on("beforeSync", ({ pack }) => {
            //console.log("enter beforeSync->pack", pack);
            //Add project id
            Object.assign(pack, {
                project_id : getParamUrl("id")
            });

            //Add ids and color for case edit
            if(pack.hasOwnProperty("tasks") && pack.tasks.hasOwnProperty("updated")){
              //Filter only tasks that need to update
              // const updatedTask = pack.tasks.updated.filter(
              //   t => t.name || t.duration || t.startDate || t.parentId || t.note || t.constrainttypeNS
              // );

              //Support update sequential
              const updatedTask = pack.tasks.updated;

              //Prevent call update forever for case not have data to update
              if(updatedTask.length === 1){
                const firstTask = updatedTask[0];
                if(!firstTask.name
                  && !firstTask.startDate
                  && !firstTask.duration
                  && !firstTask.parentId
                  && !firstTask.note
                  && !firstTask.constrainttypeNS
                  && !firstTask.afterId && !pack.hasOwnProperty("dependencies")
                ){
                  return false;
                }
              }

              updatedTask.forEach((item, i) => {
                const task = gantt.taskStore.getById(item.id);

                //Add id for task after to support insert before in NS
                item.afterId = updatedTask[i+1] ? updatedTask[i+1].id : "";
                if(syncIds.indexOf(item.id) < 0){
                  syncIds.push(item.id);

                  changeTaskColor(task, "grey");
                }
              });
            }
         });

         //After sync refresh NS's task, change color
         gantt.project.on("sync", ({ response }) => {
           if(response){
             //Refresh parent tasks
             if(window.hasOwnProperty("opener") && window.opener){
               window.opener.ShowTab("communication", true);
               window.opener.ShowTab("schedule", true);
             }

             if(response.error_msg){
               //Toast.show(response.error_msg);

               //Remove new task out
               if(response.add_fail && response.tasks && response.tasks.rows && response.tasks.rows[0]["$PhantomId"])
                  gantt.taskStore.remove(response.tasks.rows[0]["$PhantomId"]);

               //Update color for edit failed
               if(response.id){
                 const task = gantt.taskStore.getById(response.id);
                 if(task){
                   changeTaskColor(task, "red");
                 }
               }
             }else if(response.msg){
               //Toast.show(response.msg);

               //Remove out syncIds & back color to normal
               if(response.ids){
                 response.ids.forEach((item, i) => {
                   const task = gantt.taskStore.getById(item);
                   if(task){
                     changeTaskColor(task, "");
                     syncIds = syncIds.filter(s => s !== item);
                   }
                 });
               }
             }
           }
         });

         //Server failed to sync, rollback chart
         gantt.project.on("syncFail", ( response ) => {
           gantt.project.stm.undo();
         });

         //Actions on task
         gantt.taskStore.on({
            //Validate before delete task
            beforeremove({ records }) {
              const { _data } = records[0];
                if (_data && !_data.canDel && _data.canDel === false) {
                  Toast.show("Can't delete project task that already started or assigned!");
                  gantt.refresh();
                  return false;
                }
            },

            //Validate add new subtask
            beforeadd({ records, parent }) {
              if (!loaded) {
                  loaded = true
              } else {
                const parentTask = gantt.taskStore.getById(parent.id);
                if(parentTask && parentTask.isLeaf && parentTask.issummarytask === "F"){
                  Toast.show("Can't add sub task because " + parentTask.name + " is not a milestone!");
                  return false;
                }
              }
            },

            //Open popup edit task after added
            add({ records}) {
              const task = records[0];
              if(task.isPhantom)
                gantt.editTask(records[0]);
            }
        });

         //Delete predecessor/successor by double click on arrow
         gantt.on("dependencyDblClick", ({ dependency }) => {
            dependency.remove();
         });
    }, []);

    //Set color for task and toggle readOnly
    const changeTaskColor = (task, color) => {
      if(gantt){
        try{
          const taskDom = gantt.getElementFromTaskRecord(task);
          taskDom.style.backgroundColor = color;
          task.readOnly = color === "grey" ? true : false;
        }catch(err){
          console.log(err);
        }
      }
    }

    //Refresh all tasks
    const refreshTasks = () => {
      //Reset all changed color
      for (let i = 0; i < syncIds.length; i++) {
        changeTaskColor(gantt.taskStore.getById(syncIds[i]), "");
      }

      gantt.project.load().then(res => {
        //Temporary call load 2 time for fixed bugs about
        //Change task shape from rectange to diamond
        //// TODO: Call only 1 load
        gantt.project.load();
      });
    }

    return (
      <>
        <Fragment>
            <div style={{textAlign: "center"}}>
              <DemoButton
                text={'Refresh'}
                onClick={() => refreshTasks()}
              />
            </div>
            <BryntumDemoHeader
                children={<BryntumThemeCombo />}
            />

            <BryntumGantt
              ref={ganttComponent}
              {...ganttConfig}
              extraData={{ handleEditClick }}
            />
        </Fragment>
      </>
    );
};

export default App;
